/*
Module : SocksRequest.h
Purpose: Defines the interface for the CSocksRequest classes

Copyright (c) 2003 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////////// Macros / Defines //////////////////////////////

#pragma once

#ifndef __SOCKSREQUEST_H__
#define __SOCKSREQUEST_H__

#ifndef SOCKSSVC_EXT_CLASS
#define SOCKSSVC_EXT_CLASS
#endif


/////////////////////////////// Defines ///////////////////////////////////////

#pragma pack(push, 1)
struct SOCK4_CONNECT_REQUEST
{
  BYTE    VN;
  BYTE    CD;
  WORD    DSTPORT;
  in_addr DSTIP;
  BYTE    USERID[1];
};

struct SOCKS4_REPLY_PACKET
{
  BYTE    VN;
  BYTE    CD;
  WORD    DSTPORT;
  in_addr DSTIP;
};

struct SOCKS5_IDENTIFIER_PACKET
{
  BYTE VER;
  BYTE NMETHODS;
  BYTE METHODS[255];
};

struct SOCKS5_METHODSELECTION_MESSAGE
{
  BYTE VER;
  BYTE METHOD;
};

struct SOCKS5_HOST_DETAILS
{
  BYTE LENGTH;
  BYTE HOST[256]; //The extra byte is to allow us to treat this value as an asciiZ string
};

struct SOCKS5_BASE_REQUEST_DETAILS
{
  BYTE VER;
  BYTE CMD;
  BYTE RSV;
  BYTE ATYP;
};

struct SOCKS5_IP4_REQUEST_DETAILS
{
  SOCKS5_BASE_REQUEST_DETAILS Base;
  in_addr                     DST_IP;
  WORD                        DSTPORT;
};

struct SOCKS5_IP6_REQUEST_DETAILS
{
  SOCKS5_BASE_REQUEST_DETAILS Base;
  in_addr6                    DST_IP6;
  WORD                        DSTPORT;
};

struct SOCKS5_HOSTNAME_REQUEST_DETAILS
{
  SOCKS5_BASE_REQUEST_DETAILS Base;
  SOCKS5_HOST_DETAILS         DST_HOST;
  WORD                        DSTPORT;
};

struct SOCKS5_USERNAME_AUTHENTICATION_REPLY
{
  BYTE VER;
  BYTE STATUS;
};
#pragma pack(pop)


/////////////////////////////// Classes ///////////////////////////////////////

//Class which represents a request from a socks client
class SOCKSSVC_EXT_CLASS CSocksConnectionRequest
{
public:
//Constructors / Destructors
  CSocksConnectionRequest();

//Member variables
  bool                     m_bSocks5Request; //true if a socks 5 request (false implies Socks 4)
  SOCK4_CONNECT_REQUEST    m_Socks4Request;  //The socks 4 request info
  CString                  m_sSocks4Ident;   //The value as sent in the USERID field in m_Socks4Request. Note that because USERID is variable length
                                             //only the first byte of the USERID will appear in m_Socks4Request. Hence the need for this field.
  SOCKS5_IDENTIFIER_PACKET m_Socks5Request;  //The socks 5 request info "m_Socks4Request.USERID"
};

class SOCKSSVC_EXT_CLASS CSocks5RequestDetails
{
public:
//Enums
  enum Type
  {
    Unknown,
    IP6,
    IP4,
    HostName
  };

//Constructors / Destructors
  CSocks5RequestDetails();

//Member variables
  Type                            m_Type;
  SOCKS5_IP4_REQUEST_DETAILS      IP_4;
  SOCKS5_IP6_REQUEST_DETAILS      IP_6;
  SOCKS5_HOSTNAME_REQUEST_DETAILS HOSTNAME;
};

class SOCKSSVC_EXT_CLASS CSocksUsernameAuthenticationDetails
{
public:
  CString sUserName;
  CString sPassword;
};

#endif //#ifndef __SOCKSREQUEST_H__
